@echo off
SETLOCAL ENABLEDELAYEDEXPANSION

REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
REM *                                                                         *
REM *             IBM (R) PerformFIBREConfiguration.cmd                       *
REM *                                                                         *
REM *  Description: Perform FIBRE Configuration                               *
REM *                                                                         *
REM * Dependencies:                                                           *
REM *               1) FibreConfigurationExitSCLICodeHandler.cmd              *
REM *               2) scli.exe + package                                     *
REM *               3) winlpcfg.exe + package                                 *
REM *               3) CheckFibrePrerequisites.cmd                            *
REM *               4) ErrorHandler.cmd                                       *
REM *               5) SearchFile.cmd                                         *
REM *                                                                         *
REM * Copyright 2008-2009.  All rights reserved.                              *
REM *                                                                         *
REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *

REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
REM *                             Initialize                                  *
REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
:Main

SET VENDOR_HBA_FOUND=false
SET EMULEX_HBA_ADAPTERN=0
SET QLOGIC_HBA_ADAPTERN=0
SET HBA_VENDOR_OUI=Unset
SET HBA_VENDOR=Unset
SET IEEE_STANDARD=Unset
SET HBA_VENDOR_UTILITY=
SET TRANSLATE_ERROR_CODE=
SET RC=0
SET RMSG=
SET ERROR_COUNT=0
SET LAST_UTIL_FAILURE_MSG="No utility execution error(s) detected"
SET LAST_VAR_SETTING_FAILURE_MSG="No variable setting error(s) detected"
SET ERROR_MSG="Unknown error"

REM /*--- Before we even do anything.  Let's set default utility locations ---*/
Set TK_SCLI_DIR_PATH=
Set TK_SCLI_UTILITY=
Set TK_WINLPCFG_DIR_PATH=
Set TK_WINLPCFG_UTILITY=

REM /*--- Let's search scli*.exe in the system drive.  Otherwise, use default. ---*/
Set Command=Call SearchFile.cmd "%SystemDrive%\%TK_Dir_SGDeploy%\%TK_Dir_SGTKWinPE%\Bin" scli*.exe

%Command%

REM /*--- If scli*.exe found, adjust location and filename. ---*/
if exist "%SearchFile_FullPath%" (

	Set TK_SCLI_DIR_PATH=%SearchFile_Dir%
	Set TK_SCLI_UTILITY=%SearchFile_Filename%
)

REM /*--- Let's search winlpcfg.exe in the system drive.  Otherwise, use default. ---*/
Set Command=Call SearchFile.cmd "%SystemDrive%\%TK_Dir_SGDeploy%\%TK_Dir_SGTKWinPE%\Bin" winlpcfg*.exe

%Command%

REM /*--- If winlpcfg.exe found, adjust location and filename. ---*/
if exist "%SearchFile_FullPath%" (

	Set TK_WINLPCFG_DIR_PATH=%SearchFile_Dir%
	Set TK_WINLPCFG_UTILITY=%SearchFile_Filename%
)

Lecho "%%t " /F /SN
Lecho "[%0] " /F /SN /LO
Lecho "- Performing FIBRE Configuration...\n" /F

REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
REM *                   Check fo Prerequisites                                *
REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
:CheckPrerequisites

SET Command=Call CheckFibrePrerequisites.cmd

%Command%

IF errorlevel 1 (

	REM /*--- Set error count coming from CheckFibrePrerequisites.cmd. ---*/
	SET ERROR_COUNT=%ERROR_COUNT%

	REM /*--- Let's display/log the message. ---*/
	SET RMSG="<<< [ %0 ] - ERROR: '%Command%' returned a non-zero error code.  Total errors detected: !ERROR_COUNT!. >>>"

	REM /*--- Let's setup the command to be executed. ---*/
	SET Command=Lecho "\n%%t !RMSG:~1,-1!\n" /F

	!Command!

	GOTO Error_LD
)

IF /I NOT "%RC%" == "0" (

	REM /*--- Set error count coming from CheckFibrePrerequisites.cmd script. ---*/
	SET ERROR_COUNT=%ERROR_COUNT%

	REM /*--- Set last detected error message in CheckFibrePrerequisites.cmd script. ---*/
	SET LAST_VAR_SETTING_FAILURE_MSG=%LAST_VAR_SETTING_FAILURE_MSG%

	REM /*--- Let's set the message to be returned. ---*/
	SET RMSG="<<< [ %0 ] - ERROR: '%Command%' returned a non-zero error code.  There were !ERROR_COUNT! errors detected by the CheckFibrePrerequisites.cmd script.  Last detected error message: [!LAST_VAR_SETTING_FAILURE_MSG:~1,-1!]. >>>"

	REM /*--- Let's setup the command to be executed. ---*/
	SET Command=Lecho "\n%%t !RMSG:~1,-1!\n" /F

	!Command!

	GOTO Error_LD
)

REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
REM *        Let's create the list of all the TK_FIBRE_* variables            *
REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
:CreateListOfTK_FIBRE_ENVs

FOR /F "tokens=*" %%A IN ('SET TK_FIBRE_') DO (

	SET __TK_FIBRE_PARAM_NAMES.%%A
)

REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
REM *          Let's get the number of HBAs to configure                      *
REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
:GetNumberOfHBAsToConfigure

REM /*--- Default HBAs to configure is 1. ---*/
SET NUMBER_OF_HBAS_TO_CONFIGURE=Unset
FOR /F "tokens=2* delims=.=" %%A IN ('SET __TK_FIBRE_PARAM_NAMES.') DO (

	IF /I "%%A" == "TK_FIBRE_COUNT" (

		SET NUMBER_OF_HBAS_TO_CONFIGURE=%%B
	)
)

REM /*--- If the TK_FIBRE_COUNT was not set. ---*/
IF /I "%NUMBER_OF_HBAS_TO_CONFIGURE%" == "Unset" (

	REM /*--- Increment error counter. ---*/
	SET /A ERROR_COUNT+=1

	REM /*--- Let's set the message to be returned. ---*/
	SET RMSG="<<< [ %0 ] - ERROR: You must set TK_FIBRE_COUNT (number of HBA Ports to configure) to a value.  Format is: TK_FIBRE_COUNT=<Number of HBA Ports to Configure>.  Total errors detected: !ERROR_COUNT!. >>>"

	REM /*--- Let's setup the command to be executed. ---*/
	SET Command=Lecho "\n%%t !RMSG:~1,-1!\n" /F

	!Command!

	GOTO Error_LD
)

REM /*--- If the TK_FIBRE_COUNT was set to zero. ---*/
IF /I "%NUMBER_OF_HBAS_TO_CONFIGURE%" == "0" (

	REM /*--- Increment error counter. ---*/
	SET /A ERROR_COUNT+=1

	REM /*--- Let's set the message to be returned. ---*/
	SET RMSG="<<< [ %0 ] - ERROR: You must set TK_FIBRE_COUNT (number of HBA Ports to configure) to a value greater than zero.  Format is: TK_FIBRE_COUNT=<Number of HBA Ports to Configure>.  Total errors detected: !ERROR_COUNT!. >>>"

	REM /*--- Let's setup the command to be executed. ---*/
	SET Command=Lecho "\n%%t !RMSG:~1,-1!\n" /F

	!Command!

	GOTO Error_LD
)

REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
REM *                    Let's now configure each HBA                         *
REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
:ConfigureHBAs

SET HBA_NUMBER=0
SET ERROR_LEVEL=0
:Loop

	SET HBA_ID=Unset
	SET DISABLE_BOOT_DEVICES=0
	SET DISABLE_BOOT_PRIM_DEVICE=Unset
	SET DISABLE_BOOT_ALT1_DEVICE=Unset
	SET DISABLE_BOOT_ALT2_DEVICE=Unset
	SET DISABLE_BOOT_ALT3_DEVICE=Unset
	SET ENABLE_BOOT_PRIM_DEVICE_TARGET_WWNN=Unset
	SET ENABLE_BOOT_PRIM_DEVICE_TARGET_WWPN=Unset
	SET ENABLE_BOOT_PRIM_DEVICE_TARGET_LUNID=Unset
	SET ENABLE_BOOT_ALT1_DEVICE_TARGET_WWNN=Unset
	SET ENABLE_BOOT_ALT1_DEVICE_TARGET_WWPN=Unset
	SET ENABLE_BOOT_ALT1_DEVICE_TARGET_LUNID=Unset
	SET ENABLE_BOOT_ALT2_DEVICE_TARGET_WWNN=Unset
	SET ENABLE_BOOT_ALT2_DEVICE_TARGET_WWPN=Unset
	SET ENABLE_BOOT_ALT2_DEVICE_TARGET_LUNID=Unset
	SET ENABLE_BOOT_ALT3_DEVICE_TARGET_WWNN=Unset
	SET ENABLE_BOOT_ALT3_DEVICE_TARGET_WWPN=Unset
	SET ENABLE_BOOT_ALT3_DEVICE_TARGET_LUNID=Unset

	REM /*--- Increment the HBA Number. ---*/
	SET /A HBA_NUMBER+=1

	REM /*--- Let's go through the list. ---*/
	FOR /F "tokens=2* delims=.=" %%A IN ('SET __TK_FIBRE_PARAM_NAMES.TK_FIBRE_!HBA_NUMBER!') DO (


		IF /I "%%A" == "TK_FIBRE_!HBA_NUMBER!_HBA_ID" (

			REM /*--- Let's set the HBA ID ---*/
			SET HBA_ID=%%B
		)

		IF /I "%%A" == "TK_FIBRE_!HBA_NUMBER!_BOOT_DISABLE" (

			FOR %%I IN (%%B) DO (

				IF /I "%%I" == "All" (

					REM /*--- Let's set the flag to true. ---*/
					SET DISABLE_BOOT_DEVICES=1

					REM /*--- Let's add the appropriate parameter values. ---*/
					SET DISABLE_BOOT_PRIM_DEVICE=prim
					SET DISABLE_BOOT_ALT1_DEVICE=alt1
					SET DISABLE_BOOT_ALT2_DEVICE=alt2
					SET DISABLE_BOOT_ALT3_DEVICE=alt3
				)

				IF /I "%%I" == "Prim" (

					REM /*--- Let's set the flag to true. ---*/
					SET DISABLE_BOOT_DEVICES=1

					REM /*--- Let's add the appropriate parameter value. ---*/
					SET DISABLE_BOOT_PRIM_DEVICE=prim
				)

				IF /I "%%I" == "Alt1" (

					REM /*--- Let's set the flag to true. ---*/
					SET DISABLE_BOOT_DEVICES=1

					REM /*--- Let's add the appropriate parameter value. ---*/
					SET DISABLE_BOOT_ALT1_DEVICE=alt1
				)

				IF /I "%%I" == "Alt2" (

					REM /*--- Let's set the flag to true. ---*/
					SET DISABLE_BOOT_DEVICES=1

					REM /*--- Let's add the appropriate parameter value. ---*/
					SET DISABLE_BOOT_ALT2_DEVICE=alt2
				)

				IF /I "%%I" == "Alt3" (

					REM /*--- Let's set the flag to true. ---*/
					SET DISABLE_BOOT_DEVICES=1

					REM /*--- Let's add the appropriate parameter value. ---*/
					SET DISABLE_BOOT_ALT3_DEVICE=alt3
				)

				IF /I "%%I" == "No" (

					REM /*--- Let's set the flag to false. ---*/
					SET DISABLE_BOOT_DEVICES=0

					REM /*--- Let's add the appropriate parameter values. ---*/
					SET DISABLE_BOOT_PRIM_DEVICE=Unset
					SET DISABLE_BOOT_ALT1_DEVICE=Unset
					SET DISABLE_BOOT_ALT2_DEVICE=Unset
					SET DISABLE_BOOT_ALT3_DEVICE=Unset
				)
			)
		)

		IF /I "%%A" == "TK_FIBRE_!HBA_NUMBER!_BOOT_PRIM" (

			IF /I NOT "!DISABLE_BOOT_PRIM_DEVICE!" == "prim" (

				SET ENABLE_BOOT_PRIM_COUNT=1
				FOR %%I IN (%%B) DO (

					REM /*--- We are expecting first parameter to be "WWNN" ---*/
					IF "!ENABLE_BOOT_PRIM_COUNT!" == "1" SET ENABLE_BOOT_PRIM_DEVICE_TARGET_WWNN=%%I

					REM /*--- We are expecting second parameter to be "WWPN" ---*/
					IF "!ENABLE_BOOT_PRIM_COUNT!" == "2" SET ENABLE_BOOT_PRIM_DEVICE_TARGET_WWPN=%%I

					REM /*--- We are expecting third parameter to be "LUN ID" ---*/
					IF "!ENABLE_BOOT_PRIM_COUNT!" == "3" SET ENABLE_BOOT_PRIM_DEVICE_TARGET_LUNID=%%I

					REM /*--- Increment counter. ---*/
					SET /A ENABLE_BOOT_PRIM_COUNT+=1
				)
			)
		)

		IF /I "%%A" == "TK_FIBRE_!HBA_NUMBER!_BOOT_ALT1" (

			IF /I NOT "!DISABLE_BOOT_ALT1_DEVICE!" == "alt1" (

				SET ENABLE_BOOT_ALT1_COUNT=1
				FOR %%I IN (%%B) DO (

					REM /*--- We are expecting first parameter to be "WWNN" ---*/
					IF "!ENABLE_BOOT_ALT1_COUNT!" == "1" SET ENABLE_BOOT_ALT1_DEVICE_TARGET_WWNN=%%I

					REM /*--- We are expecting second parameter to be "WWPN" ---*/
					IF "!ENABLE_BOOT_ALT1_COUNT!" == "2" SET ENABLE_BOOT_ALT1_DEVICE_TARGET_WWPN=%%I

					REM /*--- We are expecting third parameter to be "LUN ID" ---*/
					IF "!ENABLE_BOOT_ALT1_COUNT!" == "3" SET ENABLE_BOOT_ALT1_DEVICE_TARGET_LUNID=%%I

					REM /*--- Increment counter. ---*/
					SET /A ENABLE_BOOT_ALT1_COUNT+=1
				)
			)
		)

		IF /I "%%A" == "TK_FIBRE_!HBA_NUMBER!_BOOT_ALT2" (

			IF /I NOT "!DISABLE_BOOT_ALT2_DEVICE!" == "alt2" (

				SET ENABLE_BOOT_ALT2_COUNT=1
				FOR %%I IN (%%B) DO (

					REM /*--- We are expecting first parameter to be "WWNN" ---*/
					IF "!ENABLE_BOOT_ALT2_COUNT!" == "1" SET ENABLE_BOOT_ALT2_DEVICE_TARGET_WWNN=%%I

					REM /*--- We are expecting second parameter to be "WWPN" ---*/
					IF "!ENABLE_BOOT_ALT2_COUNT!" == "2" SET ENABLE_BOOT_ALT2_DEVICE_TARGET_WWPN=%%I

					REM /*--- We are expecting third parameter to be "LUN ID" ---*/
					IF "!ENABLE_BOOT_ALT2_COUNT!" == "3" SET ENABLE_BOOT_ALT2_DEVICE_TARGET_LUNID=%%I

					REM /*--- Increment counter. ---*/
					SET /A ENABLE_BOOT_ALT2_COUNT+=1
				)
			)
		)

		IF /I "%%A" == "TK_FIBRE_!HBA_NUMBER!_BOOT_ALT3" (

			IF /I NOT "!DISABLE_BOOT_ALT3_DEVICE!" == "alt3" (

				SET ENABLE_BOOT_ALT3_COUNT=1
				FOR %%I IN (%%B) DO (

					REM /*--- We are expecting first parameter to be "WWNN" ---*/
					IF "!ENABLE_BOOT_ALT3_COUNT!" == "1" SET ENABLE_BOOT_ALT3_DEVICE_TARGET_WWNN=%%I

					REM /*--- We are expecting second parameter to be "WWPN" ---*/
					IF "!ENABLE_BOOT_ALT3_COUNT!" == "2" SET ENABLE_BOOT_ALT3_DEVICE_TARGET_WWPN=%%I

					REM /*--- We are expecting third parameter to be "LUN ID" ---*/
					IF "!ENABLE_BOOT_ALT3_COUNT!" == "3" SET ENABLE_BOOT_ALT3_DEVICE_TARGET_LUNID=%%I

					REM /*--- Increment counter. ---*/
					SET /A ENABLE_BOOT_ALT3_COUNT+=1
				)
			)
		)
	)

	IF /I "!HBA_ID!" == "Unset" (

		REM /*--- Increment error counter. ---*/
		SET /A ERROR_COUNT+=1

		REM /*--- Set the last variable setting failure error message. ---*/
		SET LAST_VAR_SETTING_FAILURE_MSG="<<< [ %0 ] - ERROR: Missing <hba instance> or <hba wwpn> value for TK_FIBRE_!HBA_NUMBER!_HBA_ID.  Format is: TK_FIBRE_!HBA_NUMBER!_HBA_ID=<hba instance> or <hba wwpn>. >>>"

		REM /*--- Let's set the message to be returned. ---*/
		SET Command=Lecho "\n%%t !LAST_VAR_SETTING_FAILURE_MSG:~1,-1!\n" /F

		!Command!

	) ELSE (

		REM /*--- Let's find out which vendor utility we need to use to configure this HBA. ---*/
		SET Command=Call FindFibreHBAVendor.cmd

		!Command!

		IF /I "!VENDOR_HBA_FOUND!" == "false" (

			REM /*--- Increment error counter. ---*/
			SET /A ERROR_COUNT+=1

			REM /*--- Set the last variable setting failure error message. ---*/
			SET LAST_VAR_SETTING_FAILURE_MSG="<<< [ %0 ] - ERROR: <hba instance> or <hba wwpn> value for TK_FIBRE_!HBA_NUMBER!_HBA_ID was not detected in the system. >>>"

			REM /*--- Let's set the message to be returned. ---*/
			SET Command=Lecho "\n%%t !LAST_VAR_SETTING_FAILURE_MSG:~1,-1!\n" /F

		) ELSE (

			REM /*--- Let's set the vendor HBA utility and exit code handler. ---*/
			IF /I "!HBA_VENDOR!" == "qlogic" (

				SET HBA_VENDOR_UTILITY=Call PerformQLogicFibreConfiguration.cmd
				SET TRANSLATE_ERROR_CODE=Call FibreConfigurationExitSCLICodeHandler.cmd
			)

			IF /I "!HBA_VENDOR!" == "emulex" (

				SET HBA_VENDOR_UTILITY=Call PerformEmulexFibreConfiguration.cmd
				SET TRANSLATE_ERROR_CODE=Call FibreConfigurationExitWINLPCFGCodeHandler.cmd
			)

			IF /I "!HBA_VENDOR!" == "brocade" (

				SET HBA_VENDOR_UTILITY=Call PerformBrocadeFibreConfiguration.cmd
				SET TRANSLATE_ERROR_CODE=Call FibreConfigurationExitXXXXCodeHandler.cmd
			)

			IF /I "!HBA_VENDOR!" == "unknown" (

				REM /*--- Increment error counter. ---*/
				SET /A ERROR_COUNT+=1

				REM /*--- Set the last variable setting failure error message. ---*/
				SET LAST_VAR_SETTING_FAILURE_MSG="<<< [ %0 ] - ERROR: Unable to locate the HBA fibre configuration utility for the given TK_FIBRE_!HBA_NUMBER!_HBA_ID <hba wwpn> value.  Check the WWPN number to make sure it is a valid number. >>>"

				REM /*--- Let's set the message to be returned. ---*/
				SET Command=Lecho "\n%%t !LAST_VAR_SETTING_FAILURE_MSG:~1,-1!\n" /F
			)
		)
	)

	REM /*--- Let's find out if we need to enable the 'prim' boot device setting. ---*/	
	SET ALREADY_EXECUTED=0

	REM /*--- Let's find out if it need to be configure boot from the first target the BIOS finds ---*/
	IF /I "!VENDOR_HBA_FOUND!" == "true" IF /I "!ENABLE_BOOT_PRIM_DEVICE_TARGET_WWNN!" == "0" (

		IF /I "!ENABLE_BOOT_PRIM_DEVICE_TARGET_WWPN!" == "0" (

			IF /I "!ENABLE_BOOT_PRIM_DEVICE_TARGET_LUNID!" == "0" (

				!HBA_VENDOR_UTILITY! prim-default-config

				IF /I NOT "!RC!" == "0" (

					REM /*--- Increment error counter. ---*/
					SET /A ERROR_COUNT+=1

					REM /*--- Execute the translator. ---*/
					!TRANSLATE_ERROR_CODE! !RC!

					REM /*--- Set the last utility failure error message. ---*/
					SET LAST_UTIL_FAILURE_MSG="<<< [ %0 ] - ERROR: '!HBA_VENDOR_UTILITY! prim-default-config' returned a non-zero error code (!RC!).  Error message is: !ERROR_MSG:~1,-1! >>>"

					REM /*--- Let's set the message to be returned. ---*/
					SET Command=Lecho "\n%%t !LAST_UTIL_FAILURE_MSG:~1,-1!\n" /F

					!Command!
				)

				REM /*--- Let's reset the flag to executed. ---*/
				SET ALREADY_EXECUTED=1
			)
		)

	)

	IF /I "!VENDOR_HBA_FOUND!" == "true" IF /I "!ALREADY_EXECUTED!" == "0" (

		REM /*--- Let's find out if the "WWNN" parameter value was set. ---*/
		IF /I NOT "!ENABLE_BOOT_PRIM_DEVICE_TARGET_WWNN!" == "Unset" (

			REM /*--- Let's find out if the "WWPN" parameter value was set. ---*/
			IF /I NOT "!ENABLE_BOOT_PRIM_DEVICE_TARGET_WWPN!" == "Unset" (

				REM /*--- Let's find out if the "LUN ID" parameter value was set. ---*/
				IF /I NOT "!ENABLE_BOOT_PRIM_DEVICE_TARGET_LUNID!" == "Unset" (

					!HBA_VENDOR_UTILITY! prim-config

					IF /I NOT "!RC!" == "0" (

						REM /*--- Increment error counter. ---*/
						SET /A ERROR_COUNT+=1

						REM /*--- Execute the translator. ---*/
						!TRANSLATE_ERROR_CODE! !RC!

						REM /*--- Set the last utility failure error message. ---*/
						SET LAST_UTIL_FAILURE_MSG="<<< [ %0 ] - ERROR: '!HBA_VENDOR_UTILITY! prim-config' returned a non-zero error code (!RC!).  Error message is: !ERROR_MSG:~1,-1! >>>"

						REM /*--- Let's set the message to be returned. ---*/
						SET Command=Lecho "\n%%t !LAST_UTIL_FAILURE_MSG:~1,-1!\n" /F

						!Command!
					)

				) ELSE (

					REM /*--- Increment error counter. ---*/
					SET /A ERROR_COUNT+=1

					REM /*--- Set the last variable setting failure error message. ---*/
					SET LAST_VAR_SETTING_FAILURE_MSG="<<< [ %0 ] - ERROR: Missing <lun id> value for TK_FIBRE_!HBA_NUMBER!_BOOT_PRIM.  Format is: TK_FIBRE_!HBA_NUMBER!_BOOT_PRIM=<target wwnn> <target wwpn> <lun id>. >>>"

					REM /*--- Let's set the message to be returned. ---*/
					SET Command=Lecho "\n%%t !LAST_VAR_SETTING_FAILURE_MSG:~1,-1!\n" /F

					!Command!
				)

			) ELSE (

				REM /*--- Increment error counter. ---*/
				SET /A ERROR_COUNT+=1

				REM /*--- Set the last variable setting failure error message. ---*/
				SET LAST_VAR_SETTING_FAILURE_MSG="<<< [ %0 ] - ERROR: Missing <target wwpn> value for TK_FIBRE_!HBA_NUMBER!_BOOT_PRIM.  Format is: TK_FIBRE_!HBA_NUMBER!_BOOT_PRIM=<target wwnn> <target wwpn> <lun id>. >>>"

				REM /*--- Let's set the message to be returned. ---*/
				SET Command=Lecho "\n%%t !LAST_VAR_SETTING_FAILURE_MSG:~1,-1!\n" /F

				!Command!
			)
		)
	)


	REM /*--- Let's find out if we need to enable the 'alt1' boot device setting. ---*/	
	SET ALREADY_EXECUTED=0

	REM /*--- Let's find out if it need to be configure boot from the first target the BIOS finds ---*/
	IF /I "!VENDOR_HBA_FOUND!" == "true" IF /I "!ENABLE_BOOT_ALT1_DEVICE_TARGET_WWNN!" == "0" (

		IF /I "!ENABLE_BOOT_ALT1_DEVICE_TARGET_WWPN!" == "0" (

			IF /I "!ENABLE_BOOT_ALT1_DEVICE_TARGET_LUNID!" == "0" (

				!HBA_VENDOR_UTILITY! alt1-default-config

				IF /I NOT "!RC!" == "0" (

					REM /*--- Increment error counter. ---*/
					SET /A ERROR_COUNT+=1

					REM /*--- Execute the translator. ---*/
					!TRANSLATE_ERROR_CODE! !RC!

					REM /*--- Set the last utility failure error message. ---*/
					SET LAST_UTIL_FAILURE_MSG="<<< [ %0 ] - ERROR: '!HBA_VENDOR_UTILITY! alt1-default-config' returned a non-zero error code (!RC!).  Error message is: !ERROR_MSG:~1,-1! >>>"
				
					REM /*--- Let's set the message to be returned. ---*/
					SET Command=Lecho "\n%%t !LAST_UTIL_FAILURE_MSG:~1,-1!\n" /F

					!Command!
				)

				REM /*--- Let's reset the flag to executed. ---*/
				SET ALREADY_EXECUTED=1
			)
		)

	)

	IF /I "!VENDOR_HBA_FOUND!" == "true" IF /I "!ALREADY_EXECUTED!" == "0" (

		REM /*--- Let's find out if the "WWNN" parameter value was set. ---*/
		IF /I NOT "!ENABLE_BOOT_ALT1_DEVICE_TARGET_WWNN!" == "Unset" (

			REM /*--- Let's find out if the "WWPN" parameter value was set. ---*/
			IF /I NOT "!ENABLE_BOOT_ALT1_DEVICE_TARGET_WWPN!" == "Unset" (

				REM /*--- Let's find out if the "LUN ID" parameter value was set. ---*/
				IF /I NOT "!ENABLE_BOOT_ALT1_DEVICE_TARGET_LUNID!" == "Unset" (

					!HBA_VENDOR_UTILITY! alt1-config

					IF /I NOT "!RC!" == "0" (

						REM /*--- Increment error counter. ---*/
						SET /A ERROR_COUNT+=1

						REM /*--- Execute the translator. ---*/
						!TRANSLATE_ERROR_CODE! !RC!

						REM /*--- Set the last utility failure error message. ---*/
						SET LAST_UTIL_FAILURE_MSG="<<< [ %0 ] - ERROR: '!HBA_VENDOR_UTILITY! alt1-config' returned a non-zero error code (!RC!).  Error message is: !ERROR_MSG:~1,-1! >>>"
				
						REM /*--- Let's set the message to be returned. ---*/
						SET Command=Lecho "\n%%t !LAST_UTIL_FAILURE_MSG:~1,-1!\n" /F

						!Command!
					)

				) ELSE (

					REM /*--- Increment error counter. ---*/
					SET /A ERROR_COUNT+=1

					REM /*--- Set the last variable setting failure error message. ---*/
					SET LAST_VAR_SETTING_FAILURE_MSG="<<< [ %0 ] - ERROR: Missing <lun id> value for TK_FIBRE_!HBA_NUMBER!_BOOT_ALT1.  Format is: TK_FIBRE_!HBA_NUMBER!_BOOT_ALT1=<target wwnn> <target wwpn> <lun id>. >>>"

					REM /*--- Let's set the message to be returned. ---*/
					SET Command=Lecho "\n%%t !LAST_VAR_SETTING_FAILURE_MSG:~1,-1!\n" /F

					!Command!
				)

			) ELSE (

				REM /*--- Increment error counter. ---*/
				SET /A ERROR_COUNT+=1

				REM /*--- Set the last variable setting failure error message. ---*/
				SET LAST_VAR_SETTING_FAILURE_MSG="<<< [ %0 ] - ERROR: Missing <target wwpn> value for TK_FIBRE_!HBA_NUMBER!_BOOT_ALT1.  Format is: TK_FIBRE_!HBA_NUMBER!_BOOT_ALT1=<target wwnn> <target wwpn> <lun id>. >>>"

				REM /*--- Let's set the message to be returned. ---*/
				SET Command=Lecho "\n%%t !LAST_VAR_SETTING_FAILURE_MSG:~1,-1!\n" /F

				!Command!
			)
		)
	)

	REM /*--- Let's find out if we need to enable the 'alt1' boot device setting. ---*/	
	SET ALREADY_EXECUTED=0

	REM /*--- Let's find out if it need to be configure boot from the first target the BIOS finds ---*/
	IF /I "!VENDOR_HBA_FOUND!" == "true" IF /I "!ENABLE_BOOT_ALT2_DEVICE_TARGET_WWNN!" == "0" (

		IF /I "!ENABLE_BOOT_ALT2_DEVICE_TARGET_WWPN!" == "0" (

			IF /I "!ENABLE_BOOT_ALT2_DEVICE_TARGET_LUNID!" == "0" (

				!HBA_VENDOR_UTILITY! alt2-default-config

				IF /I NOT "!RC!" == "0" (

					REM /*--- Increment error counter. ---*/
					SET /A ERROR_COUNT+=1

					REM /*--- Execute the translator. ---*/
					!TRANSLATE_ERROR_CODE! !RC!

					REM /*--- Set the last utility failure error message. ---*/
					SET LAST_UTIL_FAILURE_MSG="<<< [ %0 ] - ERROR: '!HBA_VENDOR_UTILITY! alt2-default-config' returned a non-zero error code (!RC!).  Error message is: !ERROR_MSG:~1,-1! >>>"

					REM /*--- Let's set the message to be returned. ---*/
					SET Command=Lecho "\n%%t !LAST_UTIL_FAILURE_MSG:~1,-1!\n" /F

					!Command!
				)
				
				REM /*--- Let's reset the flag to executed. ---*/
				SET ALREADY_EXECUTED=1
			)
		)

	)

	IF /I "!VENDOR_HBA_FOUND!" == "true" IF /I "!ALREADY_EXECUTED!" == "0" (

		REM /*--- Let's find out if the "WWNN" parameter value was set. ---*/
		IF /I NOT "!ENABLE_BOOT_ALT2_DEVICE_TARGET_WWNN!" == "Unset" (

			REM /*--- Let's find out if the "WWPN" parameter value was set. ---*/
			IF /I NOT "!ENABLE_BOOT_ALT2_DEVICE_TARGET_WWPN!" == "Unset" (

				REM /*--- Let's find out if the "LUN ID" parameter value was set. ---*/
				IF /I NOT "!ENABLE_BOOT_ALT2_DEVICE_TARGET_LUNID!" == "Unset" (

					!HBA_VENDOR_UTILITY! alt2-config

					IF /I NOT "!RC!" == "0" (

						REM /*--- Increment error counter. ---*/
						SET /A ERROR_COUNT+=1

						REM /*--- Execute the translator. ---*/
						!TRANSLATE_ERROR_CODE! !RC!

						REM /*--- Set the last utility failure error message. ---*/
						SET LAST_UTIL_FAILURE_MSG="<<< [ %0 ] - ERROR:'!HBA_VENDOR_UTILITY! alt2-config' returned a non-zero error code (!RC!).  Error message is: !ERROR_MSG:~1,-1! >>>"

						REM /*--- Let's set the message to be returned. ---*/
						SET Command=Lecho "\n%%t !LAST_UTIL_FAILURE_MSG:~1,-1!\n" /F

						!Command!
					)					

				) ELSE (

					REM /*--- Increment error counter. ---*/
					SET /A ERROR_COUNT+=1

					REM /*--- Set the last variable setting failure error message. ---*/
					SET LAST_VAR_SETTING_FAILURE_MSG="<<< [ %0 ] - ERROR: Missing <lun id> value for TK_FIBRE_!HBA_NUMBER!_BOOT_ALT2.  Format is: TK_FIBRE_!HBA_NUMBER!_BOOT_ALT2=<target wwnn> <target wwpn> <lun id>. >>>"

					REM /*--- Let's set the message to be returned. ---*/
					SET Command=Lecho "\n%%t !LAST_VAR_SETTING_FAILURE_MSG:~1,-1!\n" /F

					!Command!
				)

			) ELSE (

				REM /*--- Increment error counter. ---*/
				SET /A ERROR_COUNT+=1

				REM /*--- Set the last variable setting failure error message. ---*/
				SET LAST_VAR_SETTING_FAILURE_MSG="<<< [ %0 ] - ERROR: Missing <target wwpn> value for TK_FIBRE_!HBA_NUMBER!_BOOT_ALT2.  Format is: TK_FIBRE_!HBA_NUMBER!_BOOT_ALT2=<target wwnn> <target wwpn> <lun id>. >>>"

				REM /*--- Let's set the message to be returned. ---*/
				SET Command=Lecho "\n%%t !LAST_VAR_SETTING_FAILURE_MSG:~1,-1!\n" /F

				!Command!
			)
		)
	)

	REM /*--- Let's find out if we need to enable the 'alt3' boot device setting. ---*/
	SET ALREADY_EXECUTED=0

	REM /*--- Let's find out if it need to be configure boot from the first target the BIOS finds ---*/
	IF /I "!VENDOR_HBA_FOUND!" == "true" IF /I "!ENABLE_BOOT_ALT3_DEVICE_TARGET_WWNN!" == "0" (

		IF /I "!ENABLE_BOOT_ALT3_DEVICE_TARGET_WWPN!" == "0" (

			IF /I "!ENABLE_BOOT_ALT3_DEVICE_TARGET_LUNID!" == "0" (

				!HBA_VENDOR_UTILITY! alt3-default-config

				IF /I NOT "!RC!" == "0" (

					REM /*--- Increment error counter. ---*/
					SET /A ERROR_COUNT+=1

					REM /*--- Execute the translator. ---*/
					!TRANSLATE_ERROR_CODE! !RC!

					REM /*--- Set the last utility failure error message. ---*/
					SET LAST_UTIL_FAILURE_MSG="<<< [ %0 ] - ERROR: '!HBA_VENDOR_UTILITY! alt3-default-config' returned a non-zero error code (!RC!).  Error message is: !ERROR_MSG:~1,-1! >>>"

					REM /*--- Let's set the message to be returned. ---*/
					SET Command=Lecho "\n%%t !LAST_UTIL_FAILURE_MSG:~1,-1!\n" /F

					!Command!
				)

				REM /*--- Let's reset the flag to executed. ---*/
				SET ALREADY_EXECUTED=1
			)
		)
	)

	IF /I "!VENDOR_HBA_FOUND!" == "true" IF /I "!ALREADY_EXECUTED!" == "0" (

		REM /*--- Let's find out if the "WWNN" parameter value was set. ---*/
		IF /I NOT "!ENABLE_BOOT_ALT3_DEVICE_TARGET_WWNN!" == "Unset" (

			REM /*--- Let's find out if the "WWPN" parameter value was set. ---*/
			IF /I NOT "!ENABLE_BOOT_ALT3_DEVICE_TARGET_WWPN!" == "Unset" (

				REM /*--- Let's find out if the "LUN ID" parameter value was set. ---*/
				IF /I NOT "!ENABLE_BOOT_ALT3_DEVICE_TARGET_LUNID!" == "Unset" (

					!HBA_VENDOR_UTILITY! alt3-config

					IF /I NOT "!RC!" == "0" (

						REM /*--- Increment error counter. ---*/
						SET /A ERROR_COUNT+=1

						REM /*--- Execute the translator. ---*/
						!TRANSLATE_ERROR_CODE! !RC!

						REM /*--- Set the last utility failure error message. ---*/
						SET LAST_UTIL_FAILURE_MSG="<<< [ %0 ] - ERROR: '!HBA_VENDOR_UTILITY! alt3-config' returned a non-zero error code (!RC!).  Error message is: !ERROR_MSG:~1,-1! >>>"
				
						REM /*--- Let's set the message to be returned. ---*/
						SET Command=Lecho "\n%%t !LAST_UTIL_FAILURE_MSG:~1,-1!\n" /F

						!Command!
					)

				) ELSE (

					REM /*--- Increment error counter. ---*/
					SET /A ERROR_COUNT+=1

					REM /*--- Set the last variable setting failure error message. ---*/
					SET LAST_VAR_SETTING_FAILURE_MSG="<<< [ %0 ] - ERROR: Missing <lun id> value for TK_FIBRE_!HBA_NUMBER!_BOOT_ALT3.  Format is: TK_FIBRE_!HBA_NUMBER!_BOOT_ALT3=<target wwnn> <target wwpn> <lun id>. >>>"

					REM /*--- Let's set the message to be returned. ---*/
					SET Command=Lecho "\n%%t !LAST_VAR_SETTING_FAILURE_MSG:~1,-1!\n" /F

					!Command!
				)

			) ELSE (

				REM /*--- Increment error counter. ---*/
				SET /A ERROR_COUNT+=1

				REM /*--- Set the last variable setting failure error message. ---*/
				SET LAST_VAR_SETTING_FAILURE_MSG="<<< [ %0 ] - ERROR: Missing <target wwpn> value for TK_FIBRE_!HBA_NUMBER!_BOOT_ALT3.  Format is: TK_FIBRE_!HBA_NUMBER!_BOOT_ALT3=<target wwnn> <target wwpn> <lun id>. >>>"

				REM /*--- Let's set the message to be returned. ---*/
				SET Command=Lecho "\n%%t !LAST_VAR_SETTING_FAILURE_MSG:~1,-1!\n" /F

				!Command!
			)
		)
	)

	IF /I "!VENDOR_HBA_FOUND!" == "true" IF /I "!DISABLE_BOOT_DEVICES!" == "1" (

		IF /I "!DISABLE_BOOT_ALT3_DEVICE!" == "alt3" (

			!HBA_VENDOR_UTILITY! alt3-disable

			IF /I NOT "!RC!" == "0" (

				REM /*--- Increment error counter. ---*/
				SET /A ERROR_COUNT+=1

				REM /*--- Execute the translator. ---*/
				!TRANSLATE_ERROR_CODE! !RC!

				REM /*--- Set the last utility failure error message. ---*/
				SET LAST_UTIL_FAILURE_MSG="<<< [ %0 ] - ERROR: '!HBA_VENDOR_UTILITY! alt3-disable' returned a non-zero error code (!RC!).  Error message is: !ERROR_MSG:~1,-1! >>>"

				REM /*--- Let's set the message to be returned. ---*/
				SET Command=Lecho "\n%%t !LAST_UTIL_FAILURE_MSG:~1,-1!\n" /F

				!Command!
			)			

			REM /*--- Let's reset the flag to executed. ---*/
			SET ALREADY_EXECUTED=1
		)

		IF /I "!DISABLE_BOOT_ALT2_DEVICE!" == "alt2" (

			!HBA_VENDOR_UTILITY! alt2-disable

			IF /I NOT "!RC!" == "0" (

				REM /*--- Increment error counter. ---*/
				SET /A ERROR_COUNT+=1

				REM /*--- Execute the translator. ---*/
				!TRANSLATE_ERROR_CODE! !RC!

				REM /*--- Set the last utility failure error message. ---*/
				SET LAST_UTIL_FAILURE_MSG="<<< [ %0 ] - ERROR: '!HBA_VENDOR_UTILITY! alt2-disable' returned a non-zero error code (!RC!).  Error message is: !ERROR_MSG:~1,-1! >>>"
				
				REM /*--- Let's set the message to be returned. ---*/
				SET Command=Lecho "\n%%t !LAST_UTIL_FAILURE_MSG:~1,-1!\n" /F

				!Command!
			)

			REM /*--- Let's reset the flag to executed. ---*/
			SET ALREADY_EXECUTED=1
		)

		IF /I "!DISABLE_BOOT_ALT1_DEVICE!" == "alt1" (

			!HBA_VENDOR_UTILITY! alt1-disable

			IF /I NOT "!RC!" == "0" (

				REM /*--- Increment error counter. ---*/
				SET /A ERROR_COUNT+=1

				REM /*--- Execute the translator. ---*/
				!TRANSLATE_ERROR_CODE! !RC!

				REM /*--- Set the last utility failure error message. ---*/
				SET LAST_UTIL_FAILURE_MSG="<<< [ %0 ] - ERROR: '!HBA_VENDOR_UTILITY! alt1-disable' returned a non-zero error code (!RC!).  Error message is: !ERROR_MSG:~1,-1! >>>"

				REM /*--- Let's set the message to be returned. ---*/
				SET Command=Lecho "\n%%t !LAST_UTIL_FAILURE_MSG:~1,-1!\n" /F

				!Command!
			)

			REM /*--- Let's reset the flag to executed. ---*/
			SET ALREADY_EXECUTED=1
		)

		IF /I "!DISABLE_BOOT_PRIM_DEVICE!" == "prim" (

			!HBA_VENDOR_UTILITY! prim-disable

			IF /I NOT "!RC!" == "0" (

				REM /*--- Increment error counter. ---*/
				SET /A ERROR_COUNT+=1

				REM /*--- Execute the translator. ---*/
				!TRANSLATE_ERROR_CODE! !RC!

				REM /*--- Set the last utility failure error message. ---*/
				SET LAST_UTIL_FAILURE_MSG="<<< [ %0 ] - ERROR: '!HBA_VENDOR_UTILITY! prim-disable' returned a non-zero error code (!RC!).  Error message is: !ERROR_MSG:~1,-1! >>>"
				
				REM /*--- Let's set the message to be returned. ---*/
				SET Command=Lecho "\n%%t !LAST_UTIL_FAILURE_MSG:~1,-1!\n" /F

				!Command!
			)

			REM /*--- Let's reset the flag to executed. ---*/
			SET ALREADY_EXECUTED=1
		)
	)

IF NOT %HBA_NUMBER%==%NUMBER_OF_HBAS_TO_CONFIGURE% GOTO Loop

REM /*--- Let's check the number of errors found. ---*/
IF /I NOT "%ERROR_COUNT%" == "0" (

	REM /*--- Let's set the message to be returned. ---*/
	SET RMSG="<<< [ %0 ] - ERROR: Unable to configure FIBRE HBA(s).  Total errors detected: %ERROR_COUNT%.  Verify your variable settings and/or HBA(s) setup.  Last Fibre utility failure message was '[%LAST_UTIL_FAILURE_MSG:~1,-1%]'. Last variable setting failure was '[%LAST_VAR_SETTING_FAILURE_MSG:~1,-1%]'. >>>"

	REM /*--- Let's setup the command to be executed. ---*/
	SET Command=Lecho "\n%%t !RMSG:~1,-1!\n" /F

	!Command!

	GOTO Error_LD
)

REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
REM *                               SUCCESS                                   *
REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
:Success

Lecho 
Lecho "%%t " /F /SN
Lecho "[%0] " /F /SN /LO
Lecho "- FIBRE Configuration completed successfully...\n" /F

ENDLOCAL

REM /*--- Set error code to be returned. ---*/
SET RC=0

REM /*--- Let's set the message to be returned. ---*/
SET RMSG=[ %0 ] - FIBRE Configuration completed successfully...

GOTO EndOfScript

REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
REM *                                                                         *
REM *                           Error Handler                                 *
REM *                                                                         *
REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
:Error

Call ErrorHandler %0

goto Error_E     
     
:Error_D

Call ErrorHandler %0 /Display

goto Error_E     

:Error_L

Call ErrorHandler %0 /Log

goto Error_E     

:Error_LD

REM /*--- Let's return to the previous directory where we started from. ---*/
popd

ENDLOCAL

REM /*--- Set error code to be returned. ---*/
SET RC=1

REM /*--- Let's set the message to be returned. ---*/
SET RMSG=*** [ %0 ] - ERROR: Unable to configure FIBRE HBA(s).  Verify your variable settings and/or HBA(s) setup in correct.  See displayed/logged messages for additional error(s) information. ***
               
Call ErrorHandler %0 /Log /Display

:Error_E


REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
REM *                                                                         *
REM *                            End Of Script                                *
REM *                                                                         *
REM * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
:EndOfScript
