'##############################################################
'#  IBM (R) TK_DiskOrder.vbs
'#
'# This script will detect if the order
'# of the disks in diskpart.exe is in reverse order when 
'# compared to BIOS.
'# 
'# Returns the disk number for first disk on the system.
'#
'# Copyright 2008-2009. All Rights Reserved
'##############################################################

'Variables declaration
Dim RC 
Dim InvOut(1000)
Dim InvOut_RC
Dim HwdOut(1000)
Dim HwdOut_RC
Dim str_NumOfDisks
Dim str_NumOfSataDisks
Dim lsisas_pos,srsas_pos,srscsi_pos,lsiscsi_pos,isrscsi_pos,isrsas_pos,srsata_pos,mrsas_pos
Dim lsisas_disks,srscsi_disks,lsiscsi_disks,srsas_disks,isrscsi_disks,isrsas_disks,srsata_disks,mrsas_disks
Dim lsi1078_pos
Dim blade_pos
Dim debug_on
Dim isBlade

'Initialized debugging
debug_on=False
'debug_on=True
isBlade=False

'Initialized return code
RC=0
Set objArgs = Wscript.Arguments
str_NumOfDisks = CInt(objArgs(0))

If Wscript.Arguments.Count = 2 Then
  str_NumOfSataDisks = CInt(objArgs(1))
End If

'Run invraid and capture output.
InvOut_RC=GetInvRAIDOutput()
If InvOut_RC<>0 Then
  WScript.Quit(RC)
End If

'Read hardware detect output file
HwdOut_RC=ReadHWFile()
If HwdOut_RC<>0 Then
   Wscript.Quit(RC)
End If

'Detect the hardware where diskpart may have a problem with disk order
lsisas_pos=DetectLSISAS()
srsas_pos=DetectSRSASAdapter()
srscsi_pos=DetectSRSCSIAdapter()
lsiscsi_pos=DetectLSISCSI()
isrscsi_pos=DetectISRCSIAdapter()
isrsas_pos=DetectISRSASAdapter()
srsata_pos=DetectSRSATAAdapter()
mrsas_pos=DetectMegaRAIDAdapter()
lsi1078_pos=DetectLSISAS1078()

'Determine the type of controllers on the system and the position of the controller. 
If debug_on Then
    Wscript.Echo "DEBUG:lsisas_pos=" & lsisas_pos
    Wscript.Echo "DEBUG:lsiscsi_pos=" & lsiscsi_pos
    Wscript.Echo "DEBUG:srsas_pos=" & srsas_pos
    Wscript.Echo "DEBUG:srscsi_pos=" & srscsi_pos
    Wscript.Echo "DEBUG:isrsas_pos=" & isrsas_pos
    Wscript.Echo "DEBUG:isrscsi_pos=" & isrscsi_pos
    Wscript.Echo "DEBUG:mrsas_pos=" & mrsas_pos
End If


'Detect if the system is a Blade
temp_value=SearchHwOut(0,"Blade_Slot")
blade_pos=CInt(GetHWValue(temp_value))
If blade_pos > -1 Then
   isBlade=True
End If


'If a blade system, check for lsi sas and serveraid sas adapter
If isBlade Then
   'If SATA disks are available, there is no disk order issue. 
   If str_NumOfSataDisks > 0  Then
	Wscript.Echo "SATA drives found on blade"
	Wscript.Quit(0)
    End If
  
   'If LSI SAS and ServeRAID-8k/8k-l are available, we need to skip the disks for ServeRAID.
   If lsisas_pos > 0 and isrsas_pos > 0 Then
  	Wscript.Echo "LSI SAS and SR SAS found"
	'Get the number of disks for LSI SAS
   	lsisas_disks=GetLSISASDisks(lsisas_pos)
   	If debug_on Then
       		Wscript.Echo "DEBUG:lsisas_disk=" & lsisas_disks
   	End If
	'If there are no disks for LSI SAS, there is no disk order issue.
        If lsisas_disks = 0 Then
           Wscript.Echo "LSISAS have 0 disks"   
           Wscript.Quit(0)
        End If
	'Get the number of disks for ServeRAID 8k/8k-l
        isrsas_disks=GetSRSASDisks(isrsas_pos)
        If debug_on Then
           Wscript.Echo "DEBUG:srsas_disks=" & isrsas_disks
        End If
	'If there are no disks for ServeRAID, there is n odisk order issue.
        If isrsas_disks = 0 Then
    	   Wscript.Echo "SRSAS have 0 disks."     
        End If
        'The number of disks to skip is the number of disks belonging to ServeRAID.
        RC=isrsas_disks
        Wscript.Echo "Returning number of srsas disks to skip=" & RC
        WScript.Quit(RC)
   End If
End If

' System has a LSI SAS and ServeRAID SAS where LSI SAS has the boot device.
If lsisas_pos > 0 and srsas_pos > 0 Then 
   ' Get the LSI SAS disks
   lsisas_disks=GetLSISASDisks(lsisas_pos)
   If debug_on Then
       Wscript.Echo "DEBUG:lsisas_disk=" & lsisas_disks
   End If
   If lsisas_disks = 0 Then
      Wscript.Echo "LSISAS have 0 disks"    
      Wscript.Quit(0)
   End If
   
    ' Get the ServeRAID SAS disks
    srsas_disks=GetSRSASDisks(srsas_pos)
    If debug_on Then
        Wscript.Echo "DEBUG:srsas_disks=" & srsas_disks
    End If
    If srsas_disks = 0 Then
    	Wscript.Echo "SRSAS have 0 disks."     
    End If

   RC=srsas_disks
   Wscript.Echo "Returning number of srsas disks to skip=" & RC
   WScript.Quit(RC)
End If


'System has external ServeRAID SCSI and internal LSI SCSI where LSI SCSI has the boot device.
If srscsi_pos > 0 and lsiscsi_pos > 0 Then
	lsiscsi_disks=GetLSISCSIDisks(srscsi_pos)
	If debug_on Then
       		Wscript.Echo "DEBUG:lsiscsi_disk=" & lsisci_disks
   	End If
   	If lsiscsi_disks = 0 Then
      		Wscript.Echo "LSISCSI have 0 disks"    
		      Wscript.Quit(0)
  	End If

	srscsi_disks= GetSRSCSIDisks(srscsi_pos)
    	If debug_on Then
        	Wscript.Echo "DEBUG:srscsi_disks=" & srscsi_disks
   	End If
    	If srscsi_disks = 0 Then
    		Wscript.Echo "SR SCSI have 0 disks."     
    	End If

   	RC=srscsi_disks
   	Wscript.Echo "Returning number of sr scsi disks to skip=" & RC
	WScript.Quit(RC)
End If

'System has internal ServeRAID SCSI and external ServeRAID SCSI where internal SCSI has the boot device.
If isrscsi_pos > 0 and srscsi_pos > 0 Then
	isrscsi_disks=GetSRCSIDisks(isrscsi_pos)
	If debug_on Then
		Wscript.Echo "DEBUG:internal scsi disks=" & isrscsi_disks
	End If
	
	If isrscsi_disks = 0 Then
	    Wscript.Echo "Internal SR SCSI have 0 disks"
	    Wscript.Quit(0)
	End If

	srscsi_disks= GetSRSCSIDisks(srscsi_pos)
    	If debug_on Then
        	Wscript.Echo "DEBUG:srscsi_disks=" & srscsi_disks
   	End If

    	If srscsi_disks = 0 Then
    	   Wscript.Echo "External SR SCSI have 0 disks."     
    	End If

   	RC=srscsi_disks
   	Wscript.Echo "Returning number of sr scsi disks to skip=" & RC
	WScript.Quit(RC)
End If

'System has onboard SATA drives and ServeRAID SAS where the boot device is the SATA drive.
If str_NumOfSataDisks > 0 and  srsas_pos > 0 then
    srsas_disks=GetSRSASDisks(srsas_pos)
    If debug_on Then
        Wscript.Echo "DEBUG:srsas_disks=" & srsas_disks
    End If
    If srsas_disks = 0 Then
    	Wscript.Echo "SRSAS have 0 disks."     
    End If

   RC=srsas_disks
   Wscript.Quit(RC)
End If

'System has onboard SATA drives and external LSI SAS
If str_NumOfSataDisks > 0 and lsisas_pos > 0 then

   lsisas_disks=GetLSISASDisks(lsisas_pos)
   If debug_on Then
       Wscript.Echo "DEBUG:lsisas_disk=" & lsisas_disks
   End If
   If lsisas_disks = 0 Then
      Wscript.Echo "LSISAS have 0 disks"    
   End If

   RC=lsisas_disks
   Wscript.Echo "Returning number of lsisas disks to skip=" & RC
   Wscript.Quit(RC)

End If

' In multinode systems with LSI 1078 and MR 10k, the boot device is located on the
' device with the lower bus number.

lsisas_num=CountAdapter("LSI-SAS-1078-IR")

'mrsas_num=CountAdapter("ServeRAID-MR10k")
If lsi1078_pos > 0 and mrsas_pos > 0 then
   'Get the bus number for both MR 10k and 1078
   mrsas_busNum=GetBusNumber(mrsas_pos)
   lsisas_busNum=GetBusNumber(lsi1078_pos)

   ' Determine which device has the lower bus number.
   If (mrsas_busNum < lsisas_busNum) then
	mrsas_disks=GetMRSASDisks(mrsas_pos)
        If mrsas_disks = 0 then
	    RC=mrsas_disks
	    Wscript.Quit(RC)
	Else
   	  disksToSkip=0
	  iter=0
	  Do While iter < lsisas_num
	     lsisas_disks=GetLSISASDisks(lsi1078_pos)
	     numOfArrays = GetNumberOfArrays(lsi1078_pos)
    		If numOfArrays = 0 then
 		   disksToSkip=disksToSkip + lsisas_disks
		Else
		   numOfDrivesInArray=GetNumberofDrivesInArray(lsi1078_pos)
    		   numOfHotspares=GetNumberOfHotspares(lsi1078_pos)    
		   usableDisks=lsisas_disks - numOfHotspares
		   disksToSkip=disksToSkip + usableDisks - numOfDrivesInArray + numOfArrays 
		End If
	  	iter=iter+1     
		lsi1078_pos=GetNextAdapter("LSI-SAS-1078-IR",lsi1078_pos)
	Loop
	  RC=disksToSkip
	    Wscript.Quit(RC)
	End If
	   
   End If
End If

'Check the system for multiple LSI SAS adapters excluding LSI 1078.
lsisas_num=CountAdapter("LSI-SAS-RAID")
lsi1078_num=CountAdapter("LSI-SAS-1078-IR")

temp_disknum=0
total_disks_to_skip=0
If lsisas_num > 1 and lsi1078_num < 1 then
' Initialize variables	
    RC=0    
    lsi_controller=0
    lsi_controller=DetectAdapter("LSI-SAS-RAID")
    iteration=0 
    'Loop through each LSI SAS adapter and retrieve all available disks.
    Do While iteration < lsisas_num
	  'Get available disk for current adapter
	  temp_disknum=GetAvailableDisksForLSI(lsi_controller)  
	   iteration=iteration+1
	   lsi_controller=GetNextAdapter("LSI-SAS-RAID", lsi_controller)
           Wscript.Echo "iteration : " & iteration

	  'If the onboard adapter has no available disks, then no disks need to be skipped.
	   'On blades, the last adapter in invraid is the onboard adapter.
	  If isBlade and iteration=lsisas_num Then
             If temp_disknum = 0 Then
		total_disks_to_skip=0
		Exit Do
	     End If
	  ElseIf iteration=1 and isBlade <> true Then
		   If temp_disknum = 0 Then
		total_disks_to_skip=0
		Exit Do
	     End If
          Else
             total_disks_to_skip=total_disks_to_skip+temp_disknum
             Wscript.Echo "Total disks to skip= " & total_disks_to_skip	     	     
          End If
    Loop   
RC=total_disks_to_skip
Wscript.Echo "Returning number of lsisas disks to skip=" & RC
Wscript.Quit(RC)
End If

'Check the system for ServeRAID-MR10i and ServeRAID-M5025 as this 
'combination has issues with disk order.
serveraid_mr10i_pos=DetectAdapter("ServeRAID-MR10i")
serveraid_m5025_pos=DetectAdapter("ServeRAID-M5025")
If serveraid_mr10i_pos > 0 and serveraid_m5025_pos > 0 then
    serveraid_mr10i_disks=GetMRSASDisks(serveraid_mr10i_pos)
    serveraid_m5025_disks=GetMRSASDisks(serveraid_m5025_pos)
    If serveraid_mr10i_disks = 0 Then
    	Wscript.Echo "MR10i have 0 disks." 
    Else
	RC=serveraid_m5025_disks
    End If

   Wscript.Quit(RC)
End If

Wscript.Quit(RC)

'##############################################################
'# End of Main Script
'##############################################################



'##############################################################
'# Start of Helper Functions
'##############################################################


Function DetectLSISAS()
DetectLSISAS=DetectAdapter("LSI-SAS-RAID")
End Function

Function DetectSRSASAdapter()
return_value=0
return_value=DetectAdapter("ServeRAID-8s")
DetectSRSASAdapter=return_value
End Function

Function DetectSRSATAAdapter()
 return_value=0
return_value=DetectAdapter("ServeRAID-8e-SATA")
   If return_value = 0 Then
	return_value=DetectAdapter("ServeRAID-7e-SATA")
   End If
DetectSRSATAAdapter=return_value
End Function

Function DetectISRSASAdapter()
return_value=0
return_value=DetectAdapter("ServeRAID-8k")
If return_value = 0 Then
   return_value=DetectAdapter("ServeRAID-8i")
End If
DetectISRSASAdapter=return_value
End Function

Function DetectLSISCSI()
DetectLSISCSI=DetectAdapter("LSI-SCSI-RAID")
End Function

Function DetectSRSCSIAdapter()
return_value=0
return_value=DetectAdapter("ServeRAID-6M")
If return_value = 0 Then
  return_value=DetectAdapter("ServeRAID-4Lx")
  If return_value = 0 Then
     return_value=DetectAdapter("ServeRAID-4Mx")
  End If
End If
DetectSRSCSIAdapter=return_value
End Function

Function DetectISRCSIAdapter()
return_value=0
return_value=DetectAdapter("ServeRAID-7k")
If return_value = 0 Then
   return_value=DetectAdapter("ServeRAID-5i")
   If return_value = 0 Then
       return_value=DetectAdapter("ServeRAID-6i")
   End If
End If
DetectISRCSIAdapter=return_value
End Function

Function DetectMegaRAIDAdapter()
DetectMegaRAIDAdapter=DetectAdapter("ServeRAID-MR10k")
End Function

Function DetectLSISAS1078()
DetectLSISAS1078=DetectAdapter("LSI-SAS-1078-IR")
End Function

Function GetLSISASDisks(ControllerNumber)
GetLSISASDisks=GetPhysicalDisks(ControllerNumber)
End Function

Function GetSRSASDisks(ControllerNumber)
GetSRSASDisks=GetLogicalDrives(ControllerNumber)
End Function

Function GetLSISCSIDisks(ControllerNumber)
GetLSISCSIDisks=GetPhysicalDisks(ControllerNumber)
End Function

Function GetSRSCSIDisks(ControllerNumber)
GetSRSCSIDisks=GetLogicalDrives(ControllerNumber)
End Function

Function GetMRSASDisks(ControllerNumber)
GetMRSASDisks=GetLogicalDrives(ControllerNumber)
End Function

Function GetPhysicalDisks(ControllerNumber)
'Returns number of Physical disks for controller
return_value=0
searchItem="RAIDController." & ControllerNumber & ".Physical"
ItemNumber=GetItemLine(0,searchItem)
If ItemNumber > 0 Then
  searchItem = "Total_Number_Of_Physicals"
  ItemNumber = GetItemLine(ItemNumber,searchItem)
  ItemValue  = GetValue(ItemNumber)
  If Not isEmpty(ItemValue) Then
    return_value=ItemValue
  End If
End If
If debug_on Then
    Wscript.Echo "DEBUG:GetPhysicalDisks=" & return_value
End If
GetPhysicalDisks=return_value
End Function

Function DetectAdapter(AdapterName)
'Returns ControllerNumber
'Return 0 = not present
'Return !=0 location
return_value=0
searchItem="Model = "&AdapterName
ItemNumber=GetItemLine(0,searchItem)
If ItemNumber > 0 Then
    line_str= InvOut(ItemNumber-1) 
    line_str=Right(line_str,Len(line_str)-InStr(line_str,"."))
    return_value=Left(line_str,Len(line_str)-InStr(line_str,"]"))
    'return_value=Left(line_str,Len(line_str)-1)
End If
If debug_on Then
    Wscript.Echo "DEBUG:DetectAdapter=" & return_value
End If
DetectAdapter=return_value
End Function

Function GetNextAdapter(AdapterName, currentNumber)
'Returns the next ControllerNumber of given adapter
'Return 0 = not present

return_value=0
searchItem="Model = "&AdapterName
ItemNumber=0

DO while return_value <= currentNumber
	ItemNumber=GetItemLine(ItemNumber,searchItem)		

	If ItemNumber > 0 Then
    	    line_str= InvOut(ItemNumber-1) 
    	    line_str=Right(line_str,Len(line_str)-InStr(line_str,"."))
    	    return_value=Left(line_str,Len(line_str)-InStr(line_str,"]"))
        Else
	      Exit Do 
	End If
	If return_value <= currentNumber then
		ItemNumber=ItemNumber + 2
	End If
loop
Wscript.Echo "GetNextAdapter is controller " & return_value
GetNextAdapter=return_value
End Function

Function CountAdapter(AdapterName)
'Returns the number of a given adapters.

return_value=0
searchItem="Model = " &AdapterName
Do While Not isEmpty(InvOut(i))
 pos=InStr(InvOut(i),searchItem)
 If pos > 0 Then
    If debug_on Then
        Wscript.Echo "DEBUG:GetItemLine: Found " & searchItem
    End If   
    return_value=return_value + 1
 End If
 i=i+1 
Loop

CountAdapter=return_value
End Function

'Count the number of drives in the invraid line.
Function CountDrivesInLine(LineNumber)
'Return the number of drives in a line.
Drives=InvOut(LineNumber)
numOfDrives=1
Do While Not isEmpty(Drives)
    length=Len(Drives)
    pos=Instr(Drives,",")
    If pos > 0 then
    	Drives=Right(Drives, length-pos)
	numOfDrives=numOfDrives + 1
    Else
	    Exit Do
    End If

Loop
CountDrivesInLine=numOfDrives
End Function


Function GetControllerNumber(PCI)
'Returns the controller number in invraid output based on PCI ID	
return_value=0
searchItem=PCI
ItemNumber=GetItemLine(0,searchItem)
If ItemNumber > 0 Then
      searchItem="RAIDController."
      ItemNumber = GetItemLine(ItemNumber,searchItem)
      raidControllerLine=InvOut(ItemNumber)
      length=Len(raidControllerLine)
      pos=Instr(raidControllerLine,".")
      If pos > 0 then
    	  raidControllerLine=Right(raidControllerLine, length-pos)
	  pos=Instr(raidControllerLine,".")
	  If pos > 0 then
	    raidControllerLine=Left(raidControllerLine,1)
	    return_value=CInt(raidControllerLine)
         End If
      End If
End If
GetControllerNumber=return_value
End Function


Function GetNumberOfArrays(ControllerNumber)
'Returns number of logical disks for controller
return_value=0
searchItem="RAIDController." & ControllerNumber & ".Array"
ItemNumber=GetItemLine(0,searchItem)
If ItemNumber > 0 Then
  searchItem = "Total_Number_Of_Arrays"
  ItemNumber = GetItemLine(ItemNumber,searchItem)
  ItemValue  = GetValue(ItemNumber)
  If Not isEmpty(ItemValue) Then
    return_value=ItemValue
  End If
End If
If debug_on Then
    Wscript.Echo "DEBUG:GetArrayDrives=" & return_value
End If
GetNumberOfArrays=return_value
End Function


Function GetLogicalDrives(ControllerNumber)
'Returns number of logical disks for controller
return_value=0
searchItem="RAIDController." & ControllerNumber & ".Logical"
ItemNumber=GetItemLine(0,searchItem)
If ItemNumber > 0 Then
  searchItem = "Total_Number_Of_Logicals"
  ItemNumber = GetItemLine(ItemNumber,searchItem)
  ItemValue  = GetValue(ItemNumber)
  If Not isEmpty(ItemValue) Then
    return_value=ItemValue
  End If
End If
If debug_on Then
    Wscript.Echo "DEBUG:GetLogicalDrives=" & return_value
End If
GetLogicalDrives=return_value
End Function


Function GetNumberofDrivesInArray(ControllerNumber)
'Returns number of logical disks for controller
return_value=0
searchItem="RAIDController." & ControllerNumber & ".Array"
ItemNumber=GetItemLine(0,searchItem)
If ItemNumber > 0 Then
  searchItem = "Total_Number_Of_Arrays"
  ItemNumber = GetItemLine(ItemNumber,searchItem)
  ItemValue  = GetValue(ItemNumber)
  If Not isEmpty(ItemValue) Then
    return_value=ItemValue
  End If
End If
If debug_on Then
    Wscript.Echo "DEBUG:GetNumberofDrivesInArray=" & return_value
End If

If return_value > 0 Then
   numofarrays=return_value
   drivesinarrays=0
   drvs=0
   For iterator=1 to numofarrays
	searchItem="Members." & iterator
	ItemNumber=GetItemLine(ItemNumber,searchItem)
	drvs=CountDrivesInLine(ItemNumber)
	drivesinarrays=drivesinarrays + drvs		
   Next
End If
GetNumberofDrivesInArray=drivesinarrays
End Function

Function GetNumberOfHotspares(ControllerNumber)
'Return the number of hotspares for a controller
return_value=0
searchItem="RAIDController." & ControllerNumber & ".Hotspares"
ItemNumber=GetItemLine(0,searchItem)
If ItemNumber > 0 Then
  searchItem = "Total_Number_Of_Hotspares"
  ItemNumber = GetItemLine(ItemNumber,searchItem)
  ItemValue  = GetValue(ItemNumber)
  If Not isEmpty(ItemValue) Then
    return_value=ItemValue
  End If
End If

GetNumberOfHotspares=return_value
End Function

Function GetBusNumber(controllernum)
'Get the bus number for a controller.
busNumber=0
searchItem="RAIDController."&controllernum
ItemNumber=GetItemLine(0,searchItem)
If ItemNumber > 0 Then
  searchItem = "PCI = "
  ItemNumber = GetItemLine(ItemNumber,searchItem)
  ItemNumber = Right(InvOut(ItemNumber), Len(InvOut(ItemNumber))-Instr(InvOut(ItemNumber),"="))
  ItemNumber=Left(ItemNumber,Instr(ItemNumber,":")-1)  
  busNumber=CInt(ItemNumber)
End If
GetBusNumber=busNumber

End Function


Function GetAvailableDisksForLSI(ControllerNumber)
'Returns all available disks for LSI controller
numOfArrays=0
numOfDrivesInArray=0
numOfHotSpares=0
return_value=0

numOfArrays=GetNumberOfArrays(ControllerNumber)
lsisas_disks=GetLSISASDisks(ControllerNumber)
If numOfArrays = 0 then
  Wscript.Echo "Returning number of lsisas disks(singles) to skip for controller " & ControllerNumber & " is" & lsisas_disks
  GetAvailableDisksForLSI=lsisas_disks
  Exit Function
End If

numOfDrivesInArray=GetNumberofDrivesInArray(ControllerNumber)
numOfHotspares=GetNumberOfHotspares(ControllerNumber)    
usableDisks=lsisas_disks - numOfHotspares
    
If numOfDrivesInArray = usableDisks then
   Wscript.Echo "Returning number of lsisas disks(arrays) to skip for controller " & ControllerNumber & " is " & numOfArrays
   GetAvailableDisksForLSI=numOfArrays
   Exit Function
End If

return_value=usableDisks - numOfDrivesInArray + numOfArrays 	 
Wscript.Echo "Returning number of lsisas disks(arrays/singles) to skip for controller  " & ControllerNumber & " is " & return_value
GetAvailableDisksForLSI=return_value
End Function 

Function GetItemLine(start,searchItem)
i=start
Do While Not isEmpty(InvOut(i))
 pos=InStr(InvOut(i),searchItem)
 If pos > 0 Then
    If debug_on Then
        Wscript.Echo "DEBUG:GetItemLine: Found " & searchItem
    End If
    Exit Do
 End If
 i=i+1 
Loop
If isEmpty(InvOut(i)) Then
 GetItemLine=-1
Else
 GetItemLine=i
End If
End Function

Function GetValue(line)
    If debug_on Then
        Wscript.Echo "DEBUG:GetValue from:" & InvOut(line)
    End If
    return_value=Right(InvOut(line),Len(InvOut(line))-InStr(InvOut(line),"="))
    return_value=Trim(return_value)    
    GetValue=return_value
End Function

Function GetInvRAIDOutput()
Dim objShell
Dim objExecObject
Dim strLine
Dim invFileRead
Dim iterWait

'check if this file exists.  This file exists in SG file structure
invFileRead=ReadInvFile("X:\sguide\temp\invraid.ini")

If invFileRead=0 Then
  GetInvRAIDOutput=0
  Exit Function
End If

' If invraid file does not exist, create it.
iterWait=0
Set objShell=Wscript.CreateObject("Wscript.shell")
'Debugging only 
'command="%comspec% /c type invout.txt"
raid_inventory_file="raid_inv.ini"
command="%comspec% /c invraid.exe /i /f:" & raid_inventory_file
Set objExecObject=objShell.exec(command)


Do While objExecObject.Status = 0
     If iterWait > 300 Then
	WScript.Echo "Warning: " & command & "exceeded 5 minutes =" & objExecObject.Status
     	GetInvRAIDOutput=1
     	Exit Function
     End If
     WScript.Sleep 100
     iterWait=iterWait+1
Loop
If objExecObject.ExitCode<> 0 Then
     WScript.Echo "Warning: " & command & " return non-zero exit code=" & objExecObject.ExitCode
     GetInvRAIDOutput=1
     Exit Function
End If

invFileRead=ReadInvFile(raid_inventory_file)

If invFileRead<> 0 Then
  GetInvRAIDOutput=1
End If
End Function


Function ReadInvFile(invraidINI)

Const ForReading = 1
Set objFSO = CreateObject("Scripting.FileSystemObject")
Set oShell = CreateObject( "WScript.Shell" )


If Not objFSO.FileExists(invraidINI) then
    ReadInvFile=1
    Exit Function
End IF

Set objTextFile = objFSO.OpenTextFile _
    (invraidINI, ForReading)
    i=0
Do Until objTextFile.AtEndOfStream
    strNextLine = objTextFile.Readline
     InvOut(i)=strNextLine
     i=i+1
Loop
ReadInvFile=0	

End Function

Function ReadHWFile()

Const ForReading = 1
Set objFSO = CreateObject("Scripting.FileSystemObject")
Set oShell = CreateObject( "WScript.Shell" )
hwdetectINI = oShell.ExpandEnvironmentStrings("%Param_File%")

If Not objFSO.FileExists(hwdetectINI) then
   hwdetectINI = oShell.ExpandEnvironmentStrings("%TK_HWDETECT_INI%")
End IF

Set objTextFile = objFSO.OpenTextFile _
    (hwdetectINI, ForReading)
    i=0
Do Until objTextFile.AtEndOfStream
    strNextLine = objTextFile.Readline
    'Wscript.Echo "line: " & strNextLine
     HwdOut(i)=strNextLine
     i=i+1
Loop
ReadHWFile=0	
End Function

Function SearchHwOut(start,searchItem)
i=start
Do While Not isEmpty(HwdOut(i))
 pos=InStr(HwdOut(i),searchItem)
 If pos > 0 Then
    If debug_on Then
        Wscript.Echo "DEBUG:GetHW Line: Found " & searchItem
    End If
    Exit Do
 End If
 i=i+1 
Loop
If isEmpty(HwdOut(i)) Then
 SearchHwOut=-1
Else
 SearchHwOut=i
End If
End Function

Function GetHWValue(line)
    If debug_on Then
        Wscript.Echo "DEBUG:GetValue from:" & HwdOut(line)
    End If
    return_value=Right(HwdOut(line),Len(HwdOut(line))-InStr(HwdOut(line),"="))
    return_value=Trim(return_value)	
   If return_value = "" Then
	    return_value = -1
    End If
    GetHWValue=return_value
End Function

